@@ -38,7 +38,7 @@
 
 int sqlcipher_cc_setup(sqlcipher_provider *p);
 
-static int sqlcipher_cc_add_random(void *ctx, void *buffer, int length) {
+static int sqlcipher_cc_add_random(void *ctx, const void *buffer, int length) {
   return SQLITE_OK;
 }
 
@@ -53,19 +53,39 @@ static const char* sqlcipher_cc_get_provider_name(void *ctx) {
 
 static const char* sqlcipher_cc_get_provider_version(void *ctx) {
 #if TARGET_OS_MAC
-  CFTypeRef version;
+  /* macOS uses com.apple.security with a lowercase s */
   CFBundleRef bundle = CFBundleGetBundleWithIdentifier(CFSTR("com.apple.security"));
-  if(bundle == NULL) {
-    return "unknown";
+  CFTypeRef bundle_ver;
+  const char *ver;
+
+  /* if the bundle wasn't identified, try secrurity with a capial S (works for iOS) */
+  if(!bundle) {
+    bundle = CFBundleGetBundleWithIdentifier(CFSTR("com.apple.Security"));
   }
-  version = CFBundleGetValueForInfoDictionaryKey(bundle, CFSTR("CFBundleShortVersionString"));
-  return CFStringGetCStringPtr(version, kCFStringEncodingUTF8);
-#else
-  return "unknown";
+
+  /* If the bundle was resolved, retrieve the bundle version key then attempt to convert it to a C string.
+   * Note that it is possible for CFStringGetCString to return NULL (this is warned against extensively in the
+   * header), so  only return a value if the conversion was successful. */
+  if(
+    bundle
+    && (bundle_ver = CFBundleGetValueForInfoDictionaryKey(bundle, kCFBundleVersionKey))
+    && (ver = CFStringGetCStringPtr(bundle_ver, kCFStringEncodingUTF8))
+  ) {
+   return ver;
+  }
+
 #endif
+  /* unable to detect the CoreCrypto version, return a fixed string "unknown" */
+  return "unknown";
 }
 
-static int sqlcipher_cc_hmac(void *ctx, int algorithm, unsigned char *hmac_key, int key_sz, unsigned char *in, int in_sz, unsigned char *in2, int in2_sz, unsigned char *out) {
+static int sqlcipher_cc_hmac(
+  void *ctx, int algorithm,
+  const unsigned char *hmac_key, int key_sz,
+  const unsigned char *in, int in_sz,
+  const unsigned char *in2, int in2_sz,
+  unsigned char *out
+) {
   CCHmacContext hmac_context;
   if(in == NULL) return SQLITE_ERROR;
   switch(algorithm) {
@@ -87,7 +107,13 @@ static int sqlcipher_cc_hmac(void *ctx, int algorithm, unsigned char *hmac_key,
   return SQLITE_OK; 
 }
 
-static int sqlcipher_cc_kdf(void *ctx, int algorithm, const unsigned char *pass, int pass_sz, unsigned char* salt, int salt_sz, int workfactor, int key_sz, unsigned char *key) {
+static int sqlcipher_cc_kdf(
+  void *ctx, int algorithm,
+  const unsigned char *pass, int pass_sz,
+  const unsigned char* salt, int salt_sz,
+  int workfactor,
+  int key_sz, unsigned char *key
+) {
   switch(algorithm) {
     case SQLCIPHER_HMAC_SHA1:
       if(CCKeyDerivationPBKDF(kCCPBKDF2, (const char *)pass, pass_sz, salt, salt_sz, kCCPRFHmacAlgSHA1, workfactor, key, key_sz) != kCCSuccess) return SQLITE_ERROR;
@@ -104,7 +130,13 @@ static int sqlcipher_cc_kdf(void *ctx, int algorithm, const unsigned char *pass,
   return SQLITE_OK; 
 }
 
-static int sqlcipher_cc_cipher(void *ctx, int mode, unsigned char *key, int key_sz, unsigned char *iv, unsigned char *in, int in_sz, unsigned char *out) {
+static int sqlcipher_cc_cipher(
+  void *ctx, int mode,
+  const unsigned char *key, int key_sz,
+  const unsigned char *iv,
+  const unsigned char *in, int in_sz,
+  unsigned char *out
+) {
   CCCryptorRef cryptor;
   size_t tmp_csz, csz;
   CCOperation op = mode == CIPHER_ENCRYPT ? kCCEncrypt : kCCDecrypt;
@@ -166,6 +198,8 @@ static int sqlcipher_cc_fips_status(void *ctx) {
 }
 
 int sqlcipher_cc_setup(sqlcipher_provider *p) {
+  p->init = NULL;
+  p->shutdown = NULL;
   p->random = sqlcipher_cc_random;
   p->get_provider_name = sqlcipher_cc_get_provider_name;
   p->hmac = sqlcipher_cc_hmac;
